/** @file
  TranslateMsrIndex function

  Copyright (c) 2013-2016 Intel Corporation.

  Redistribution and use in source and binary forms, with or without
  modification, are permitted provided that the following conditions
  are met:

  * Redistributions of source code must retain the above copyright
  notice, this list of conditions and the following disclaimer.
  * Redistributions in binary form must reproduce the above copyright
  notice, this list of conditions and the following disclaimer in
  the documentation and/or other materials provided with the
  distribution.
  * Neither the name of Intel Corporation nor the names of its
  contributors may be used to endorse or promote products derived
  from this software without specific prior written permission.

  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

**/

#include <QuarkNcSocId.h>
#include "QuarkMsr.h"
#include <MtrrLib.h>

//
// This table defines the offset, base and length of the fixed MTRRs
//
STATIC
QUARK_MTRR_CONVERSION_TABLE    QuarkMtrrConversionTable[] = {
  {MTRR_LIB_IA32_MTRR_CAP, QUARK_NC_HOST_BRIDGE_IA32_MTRR_CAP},
  {MTRR_LIB_IA32_MTRR_DEF_TYPE, QUARK_NC_HOST_BRIDGE_IA32_MTRR_DEF_TYPE},
  {MTRR_LIB_IA32_MTRR_FIX64K_00000, QUARK_NC_HOST_BRIDGE_MTRR_FIX64K_00000},
  {MTRR_LIB_IA32_MTRR_FIX16K_80000, QUARK_NC_HOST_BRIDGE_MTRR_FIX16K_80000},
  {MTRR_LIB_IA32_MTRR_FIX16K_A0000, QUARK_NC_HOST_BRIDGE_MTRR_FIX16K_A0000},
  {MTRR_LIB_IA32_MTRR_FIX4K_C0000, QUARK_NC_HOST_BRIDGE_MTRR_FIX4K_C0000},
  {MTRR_LIB_IA32_MTRR_FIX4K_C8000, QUARK_NC_HOST_BRIDGE_MTRR_FIX4K_C8000},
  {MTRR_LIB_IA32_MTRR_FIX4K_D0000, QUARK_NC_HOST_BRIDGE_MTRR_FIX4K_D0000},
  {MTRR_LIB_IA32_MTRR_FIX4K_D8000, QUARK_NC_HOST_BRIDGE_MTRR_FIX4K_D8000},
  {MTRR_LIB_IA32_MTRR_FIX4K_E0000, QUARK_NC_HOST_BRIDGE_MTRR_FIX4K_E0000},
  {MTRR_LIB_IA32_MTRR_FIX4K_E8000, QUARK_NC_HOST_BRIDGE_MTRR_FIX4K_E8000},
  {MTRR_LIB_IA32_MTRR_FIX4K_F0000, QUARK_NC_HOST_BRIDGE_MTRR_FIX4K_F0000},
  {MTRR_LIB_IA32_MTRR_FIX4K_F8000, QUARK_NC_HOST_BRIDGE_MTRR_FIX4K_F8000},
  {EFI_MSR_PENTIUM_SMRR_PHYS_BASE, QUARK_NC_HOST_BRIDGE_IA32_MTRR_SMRR_PHYSBASE},
  {EFI_MSR_PENTIUM_SMRR_PHYS_MASK, QUARK_NC_HOST_BRIDGE_IA32_MTRR_SMRR_PHYSMASK},
  {MTRR_LIB_IA32_MTRR_PHYSBASE0, QUARK_NC_HOST_BRIDGE_IA32_MTRR_PHYSBASE0},
  {MTRR_LIB_IA32_MTRR_PHYSMASK0, QUARK_NC_HOST_BRIDGE_IA32_MTRR_PHYSMASK0},
  {MTRR_LIB_IA32_MTRR_PHYSBASE1, QUARK_NC_HOST_BRIDGE_IA32_MTRR_PHYSBASE1},
  {MTRR_LIB_IA32_MTRR_PHYSMASK1, QUARK_NC_HOST_BRIDGE_IA32_MTRR_PHYSMASK1},
  {MTRR_LIB_IA32_MTRR_PHYSBASE2, QUARK_NC_HOST_BRIDGE_IA32_MTRR_PHYSBASE2},
  {MTRR_LIB_IA32_MTRR_PHYSMASK2, QUARK_NC_HOST_BRIDGE_IA32_MTRR_PHYSMASK2},
  {MTRR_LIB_IA32_MTRR_PHYSBASE3, QUARK_NC_HOST_BRIDGE_IA32_MTRR_PHYSBASE3},
  {MTRR_LIB_IA32_MTRR_PHYSMASK3, QUARK_NC_HOST_BRIDGE_IA32_MTRR_PHYSMASK3},
  {MTRR_LIB_IA32_MTRR_PHYSBASE4, QUARK_NC_HOST_BRIDGE_IA32_MTRR_PHYSBASE4},
  {MTRR_LIB_IA32_MTRR_PHYSMASK4, QUARK_NC_HOST_BRIDGE_IA32_MTRR_PHYSMASK4},
  {MTRR_LIB_IA32_MTRR_PHYSBASE5, QUARK_NC_HOST_BRIDGE_IA32_MTRR_PHYSBASE5},
  {MTRR_LIB_IA32_MTRR_PHYSMASK5, QUARK_NC_HOST_BRIDGE_IA32_MTRR_PHYSMASK5},
  {MTRR_LIB_IA32_MTRR_PHYSBASE6, QUARK_NC_HOST_BRIDGE_IA32_MTRR_PHYSBASE6},
  {MTRR_LIB_IA32_MTRR_PHYSMASK6, QUARK_NC_HOST_BRIDGE_IA32_MTRR_PHYSMASK6},
  {MTRR_LIB_IA32_MTRR_PHYSBASE7, QUARK_NC_HOST_BRIDGE_IA32_MTRR_PHYSBASE7},
  {MTRR_LIB_IA32_MTRR_PHYSMASK7, QUARK_NC_HOST_BRIDGE_IA32_MTRR_PHYSMASK7},
  {0xFFFFFFFF, 0xFF}
};


/**
  Translates an MSR to the Quark Host Bridge equivalent MSR register.

  @param  Index The 32-bit MSR index to translate.

  @return The 8 bit translated MSR location in Host Bridge.

**/
UINT8
EFIAPI
TranslateMsrIndex (
  IN UINT32  Index
  )
{
  UINT8  QuarkMsrIndex = 0;
  UINT8  MsrTableEntry = 0;

  while(QuarkMtrrConversionTable[MsrTableEntry].StandardMsr != 0xFFFFFFFF) {
    if (Index == QuarkMtrrConversionTable[MsrTableEntry].StandardMsr) {
      QuarkMsrIndex = QuarkMtrrConversionTable[MsrTableEntry].QuarkMsr;
	  break;
    }
  MsrTableEntry++;
  }

  return(QuarkMsrIndex);
}

